<?php
/**
*
* Copyright (C) 2011 Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
*
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU Affero General Public License version 3 as published by
* the Free Software Foundation with the addition of the following permission
* added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
* WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
* WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
* details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program; if not, see www.gnu.org/licenses or write to
* the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
* MA 02110-1301 USA.
*
* This code is based on the OpenXchange Connector and on the Prognus pSync
* Connector both developed by the community and licensed under the GPL
* version 2 or above as published by the Free Software Foundation.
*
* You can contact Prognus Software Livre headquarters at Av. Tancredo Neves,
* 6731, PTI, Bl. 05, Esp. 02, Sl. 10, Foz do Iguaçu - PR - Brasil or at
* e-mail address prognus@prognus.com.br.
*
* Descrição rápida do arquivo
*
* Arquivo responsável pela manipulação dos filtros
*
* @package    filters
* @license    http://www.gnu.org/copyleft/gpl.html GPL
* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
* @version    1.0
* @sponsor    Caixa Econômica Federal
* @since      Arquivo disponibilizado na versão 2.4
*/



/**
* Classe responsável pela manipulação dos filtros.
*
*
* @package    prototype
* @license    http://www.gnu.org/copyleft/gpl.html GPL
* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
* @author     Airton Bordin Junior
* @author     Gustavo Pereira dos Santos
* @version    1.0
* @since      Classe disponibilizada na versão 2.4
*/
class FilterMapping
{
	
	var $service;
	
	
	/**
	* Método que formata o Script de acordo com a sintaxe do Sieve.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <Array> <$rules> <Array com as regras do usuário>
	* @return     <Regra de acordo com a sintaxe do Sieve>
	* @access     <public>
	*/
	public function formatSieveScript( $rules )
    {
	
		$require_fileinto = $require_flag = $require_reject = $require_vacation = $require_body = $require_imapflag = $vacation = $startswith = $endswith = false;

		$script_rules = $script_header = $script_criteria = $vacation_action = "";

		$i = 0;

		foreach( $rules as $name => $data )
		{
			/* Usado na opção Bloquear usuário do ExpressoMail */
			if($data['block']) {
				($i >0) ? $script_match = 'elsif anyof' : $script_match = 'if anyof';
				$script_match = $script_match . "(address :is \"from\" [\"" .$data['name'] . "\"]) {\r\n";  
				$script_match .= "fileinto \"INBOX/Spam\"; \r\n}\r\n";
				$script_rules .= $script_match;
				$script_match = "";
				$script_criteria = "";
				$require_fileinto = true;
				$i++;
				continue;
			}
			
			if( $data['enabled'] == 'false' )
				continue;

			$vacation = false;
			$criteria = $data['criteria'];
			$action   = $data['actions'];
			
			($i >0) ? $script_match = 'els' : $script_match = '';
			$data['isExact'] == 'false' ?  $script_match .= 'if anyof (' : $script_match .= 'if allof (';

			for ($j=0; $j<count($criteria); $j++)
			{					
				switch($criteria[$j]['field']) {
					case 'To':    
					case 'to':    
					case 'CC':
					case 'Cc':
						$criteria[$j]['field'] = "[\"To\", \"TO\", \"Cc\", \"CC\"]"; 
						$script_criteria .= "address :";
						break;
					case 'from':
						$criteria[$j]['field'] = "\"" . $criteria[$j]['field'] . "\"";
						$script_criteria .= "address :";
						break;
					case 'size':	
						$criteria[$j]['field'] = '';
						$script_criteria .= "size :";
						break;
					case 'subject':
						$criteria[$j]['field'] = "\"" . $criteria[$j]['field'] . "\"";
						$criteria[$j]['value'] = "" . imap_8bit($criteria[$j]['value']) . "\", \"" . base64_encode($criteria[$j]['value']) . "";
						$script_criteria .= "header :";
						break;
					case 'body':
						$criteria[$j]['field'] = '';
						$script_criteria .= "body :";
						$require_body = true;
						break;
					case 'vacation':
						continue;
					default:
						$script_criteria .= "header :";
						break;
				}
				
				switch ($criteria[$j]['operator']) {
					case '>':
						$criteria[$j]['operator'] = "over";
						break;
					case '<':
						$criteria[$j]['operator'] = "under";
						break;
					case '=':
						$criteria[$j]['operator'] = "is";
						$criteria[$j]['value'] = "[\"" . $criteria[$j]['value'] . "\"]";
						break;
					case '*':
						$criteria[$j]['operator'] = "contains";
						$criteria[$j]['value'] = "[\"" . $criteria[$j]['value'] . "\"]";
						break;						
					case '^':
						$criteria[$j]['operator'] = "matches";
						$criteria[$j]['value'] = "[\"" . $criteria[$j]['value'] . "*\"]";
						$startswith = true;
						break;
					case '$':
						$criteria[$j]['operator'] = "matches";
						$criteria[$j]['value'] = "[\"*" . $criteria[$j]['value'] . "\"]";
						$endswith = true;
						break;
					/* 
					   TO-DO:
					   Arrrumar regra do "não contém".
					*/
					case '!*':
						$criteria[$j]['operator'] = "contains";
						$criteria[$j]['value'] = "[\"" . $criteria[$j]['value'] . "\"]";
						break;
				}
				
				if ($criteria[$j]['field'] == "" || $criteria[$j]['field'] == "\"subject\"" || $startswith || $endswith)
				{
					$script_criteria .= $criteria[$j]['operator'] . " " . $criteria[$j]['field'] . " " . $criteria[$j]['value'] . ", "; 
					$startswith = $endswith = false;
				}
				else
					$script_criteria .= $criteria[$j]['operator'] . " " . $criteria[$j]['field'] . " " . $criteria[$j]['value'] . ", ";
			}
			$script_criteria = substr($script_criteria,0,-2);
			/* if ($vacation == false) */ 
			$script_criteria .= ")"; 

			$script_action = " {\r\n ";
			for ($k=0; $k<count($action); $k++)
			{
				switch ($action[$k]['type']) {
					case 'redirect':
						break;
					case 'reject':
						$require_reject = true;
						break;
					case 'fileinto':
						$require_fileinto = true;
						break;
					case 'vacation':
						$require_vacation = true;
						$action[$k]['parameter'] = "\"" . $action[$k]['parameter'] . "\"";
						$vacation_action = " :subject \"Fora do Escritório\" " . $action[$k]['parameter'] . ";";
						$vacation = true;
						continue;
					case 'setflag':
						$require_flag = true;
						$action[$k]['parameter'] = "\\\\" . $action[$k]['parameter'];
						break;
					case 'discard':
						break;
				}
				if ($vacation == false) $script_action .= $action[$k]['type'] . " \"" . $action[$k]['parameter'] . "\";\r\n ";
			}

			$script_action .= "}";
			
			if($vacation != true)
				$script_rules .= $script_match . $script_criteria . $script_action . "\r\n";

			if($data['id'] != "vacation")
				$i++;
			$script_match = "";
			$script_criteria = "";
			
		}

		if($require_reject || $require_fileinto || $require_vacation || $require_body || $require_flag)
		{
			$script_header .= "require [";
			$require_reject ? $script_header .= "\"reject\", " : ""; 
			$require_fileinto ? $script_header .= "\"fileinto\", " : ""; 
			$require_vacation? $script_header .= "\"vacation\", " : "";  
			$require_flag ? $script_header .= "\"imapflags\", " : "";  
			$require_body ? $script_header .= "\"body\", " : "";  /* tem que instalar as extensões no Cyrus */
			$script_header = substr($script_header,0,-2);
			$script_header .= "];\r\n";
		}

		if( $vacation_action )
		{
		  $script_rules .= "vacation" . $vacation_action . "\r\n";
		}

		$json_data = json_encode($rules);
		
		$script_begin = "#Filtro gerado por Expresso Livre versão 2.4\r\n\r\n";

		$content = $script_begin . $script_header . $script_rules . "\r\n\r\n#PseudoScript#" . "\r\n#" . $json_data;
		//$content = ""; 
		return( $content );
	}
	
	/**
	* Método que lê e faz o parser dos filtros antigos
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <$scriptName> <Regras do usuário>
	* @return     <Regra do usuário parseada>
	* @access     <public>
	*/
	public function readOldScript($scriptName) 
	{
        // Recebe o conteúdo do array;
        $lines = array();
        $lines = preg_split("/\n/", $scriptName);

        // Pega o tamanho da regra na primeira do script;
        $size_rule = array_shift($lines);

        // Recebe o tamanho do script, pela primeira linha;
        $this->size = trim($size_rule);

        // Verifica a composição do script; */
         $line = array_shift($lines);

        // Variaveis para a regra e o campo ferias;
        $regexps = array('^ *##PSEUDO', '^ *#rule', '^ *#vacation', '^ *#mode');
        $retorno['rule'] = array();

        $line = array_shift($lines);
        while (isset($line)) {
            foreach ($regexps as $regp) {
                if (preg_match("/$regp/i", $line)) {
                    // Recebe todas as regras criadas no servidor;
                    if (preg_match("/^ *#rule&&/i", $line)) {
                        $retorno['rule'][] = $line . "\n";
                    }
                }
            }
            // Pega a proxima linha do sript;
            $line = array_shift($lines);
        }
        return $retorno;
    }
	
	
	
	
	/**
	* Método que faz o parsing do Script Sieve, transformando em Array.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <String> <$script> <Script Sieve com as regras do usuário>
	* @return     <Regras do usuário em Array>
	* @access     <public>
	*/
	public function parseSieveScript( $script )
	{
		$old_rule = strripos($script, "##PSEUDO script start");
		
		if($old_rule) {
			
			$parsed_rule = $this->readOldScript($old_script);
			$array_rule = explode("&&", $parsed_rule['rule'][0]);

			$old_retorno = array();
			$old_retorno['isExact'] = false;
			$old_retorno['name'] = $array_rule[1];
			$old_retorno['criteria'] = array();
			$old_retorno['criteria'][0] = array();
			$old_retorno['criteria'][0]['value'] = $array_rule[5];
			$old_retorno['criteria'][0]['operator'] = '=';
			$old_retorno['criteria'][0]['field'] = 'subject';
			$old_retorno['actions'] = array();
			$old_retorno['actions'][0] = array();
			$old_retorno['actions'][0]['parameter'] = $array_rule[7];
			$old_retorno['actions'][0]['type'] = $array_rule[6];
			$old_retorno['enabled'] = true;
			$old_retorno['id'] = $array_rule[1];
			$old_retorno['applyMessages'] = '';

			return $old_retorno;
		}

		$pos = strripos($script, "#PseudoScript#");
		$pseudo_script = substr( $script, $pos+17 );

		$return = json_decode( $pseudo_script, true );

		return $return;
	}

	var $rules = false;

	/**
	* Construtor da classe.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @access     <public>
	*/
	public function __construct()
	{ 
		$this->service = Controller::service("Sieve");
	}

	
	/**
	* Método que recupera as regras do usuário.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @return     <Regras do usuário>
	* @access     <public>
	*/
	public function getRules()
	{
		$this->rules = Controller::find( array( 'concept' => 'filter' ) );

		if( !$this->rules ) {
			$this->rules = array();
		}
		return( $this->rules );
	}

	
	/**
	* Método que aplica o filtro para as mensagens do usuário.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <$uri> 
	* @param      <$result> 
	* @param      <$criteria> 
	* @param      <$original> 
	* @access     <public>
	*/
	public function applySieveFilter( &$uri , &$result , &$criteria , $original  )
	{
		 $result['id'] = $uri['id'];

		 if( $original['properties']['applyMessages'] )
		 {
		      $filter = Controller::read($uri);

		      $proc = array();
		      $proc['keep'] = false;
		      
		      foreach ($filter['actions'] as $v){
			      if($v['type'] == 'keep')
				      $proc['keep'] = true;
			      else{
				      $proc['type'] = $v['type'];
				      $proc['parameter'] = $v['parameter'];
			      }
		      }

		      $imap = Controller::service( 'Imap' );
		      $imap->apliSieveFilter($original['properties']['applyMessages'] , $proc ); 
		      return $result;
		  }
	}

	
	/**
	* Método que lê o script do usuário.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <$uri> 
	* @param      <$result> 
	* @param      <$criteria> 
	* @param      <$original> 
	* @return     <Script do usuário>
	* @access     <public>
	*/
	public function readUserScript( &$uri , &$params , &$criteria , $original )
	{  
		$uri['id'] = $this->service->config['user'];
	}
  
  
	/**
	* Método que seta o script do usuário.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <$uri> 
	* @param      <$result> 
	* @param      <$criteria> 
	* @param      <$original> 
	* @return     <Script do usuário>
	* @access     <public>
	*/
	public function setRule( &$uri , &$params , &$criteria , $original  )
	{
		if( !$this->rules )
		$this->rules = $this->getRules();

	    $uri['id'] = $params['id'] = isset($params['id']) ? $params['id'] : urlencode($params['name']);

	    $i = 0;

	    for( ; isset($this->rules[$i]) && $this->rules[$i]['id'] !== $params['id']; $i++ );

	    $this->rules[$i] = array_merge( ( isset($this->rules[$i]) ? $this->rules[$i] : array() ), $params );

	    $params = array( 'name' => $this->service->config['user'],
			     'content' => $this->formatSieveScript( $this->rules ),
			     'active' => true );
	}

	
	/**
	* Método que deleta o script do usuário.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <$uri> 
	* @param      <$result> 
	* @param      <$criteria> 
	* @param      <$original> 
	* @access     <public>
	*/
	public function deleteRule( &$uri, &$params, &$criteria, $original )
	{
		if( !$this->rules ) {	
			$this->rules = $this->getRules();
		}	  
		$params['id'] = $uri['id'];

		$update = false;
		
		$rules = array();

		foreach( $this->rules as $i => $rule )
			if( $rule['id'] !== $uri['id'] )
				$rules[] = $this->rules[$i];

		$this->rules = $rules;
		
		$uri['id'] = '';

		$params = array( 'name' => $this->service->config['user'],
			   'content' => $this->formatSieveScript( $this->rules ),
			   'active' => true );

		$URI = Controller::URI( $uri['concept'], $this->service->config['user'] );
		$this->service->update( $URI, $params );
	
		return( false );
	}

	
	/**
	* Método que pega o script do usuário.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <$uri> 
	* @param      <$result> 
	* @param      <$criteria> 
	* @param      <$original> 
	* @return     <Script do usuário>
	* @access     <public>
	*/
	public function getSieveRule( &$uri , &$params , &$criteria , $original )
	{	  
		$script = $this->parseSieveScript( $params['content'] );

		foreach( $script as $i => $rule )
			if( $rule['name'] === $original['id'] )
				return( $params = $rule );
	}

	
	/**
	* Método que lista as regras do usuário.
	*
	* @license    http://www.gnu.org/copyleft/gpl.html GPL
	* @author     Consórcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	* @sponsor    Caixa Econômica Federal
	* @author     Airton Bordin Junior
	* @author	  Gustavo Pereira dos Santos	
	* @param      <$uri> 
	* @param      <$result> 
	* @param      <$criteria> 
	* @param      <$original> 
	* @return     <Regras do usuário>
	* @access     <public>
	*/
	public function listSieveRules( &$uri , &$params , &$criteria , $original  )
	{
		$return = $params = $this->parseSieveScript( $params[0]['content'] ); 
		return( $return );
	}
}
